// LICENSE : MIT
"use strict";
import { moduleInterop } from "@textlint/module-interop";
import { normalizeTextlintPresetSubRuleKey } from "@textlint/utils";
import { isPresetRuleKey } from "./config-util";
/**
 * Convert config of preset to rawRulesConfig flat path format.
 *
 * This function convert Preset nesting rule to flat path
 * ```
 * {
 *  "x" : true
 *  "preset-a" : { "rule-name": "value" }
 * }
 * ```
 * =>
 * ```
 * { "x": true }
 * { "a/rule-name": "value" }
 * ```
 *
 * @param rawRulesConfig
 * @returns {{string: string}}
 */
export function createFlatRulesConfigFromRawRulesConfig(rawRulesConfig) {
    if (!rawRulesConfig) {
        return {};
    }
    var rulesConfig = {};
    Object.keys(rawRulesConfig).forEach(function (key) {
        if (isPresetRuleKey(key)) {
            // <preset>/<rule>
            var presetName = key;
            var presetRuleConfig = rawRulesConfig[key];
            Object.assign(rulesConfig, createFlatPresetRulesConfigFromRawPresetRuleConfig(presetRuleConfig, presetName));
            return;
        }
        rulesConfig[key] = rawRulesConfig[key];
    });
    return rulesConfig;
}
/**
 * create flat `<plugin>/<rule>` option
 * @param {Object} [rulesConfig]
 * @param {string} presetName
 * @returns {Object}
 */
export function createFlatPresetRulesConfigFromRawPresetRuleConfig(rulesConfig, presetName) {
    var mapped = {};
    // missing "rulesConfig"
    if (rulesConfig === undefined || typeof rulesConfig !== "object") {
        return mapped;
    }
    Object.keys(rulesConfig).forEach(function (ruleName) {
        var normalizedKey = normalizeTextlintPresetSubRuleKey({ preset: presetName, rule: ruleName });
        mapped[normalizedKey] = rulesConfig[ruleName];
    });
    return mapped;
}
// load rulesConfig from plugins
/**
 *
 * @param presetNames
 * @param {TextLintModuleResolver} moduleResolver
 * @returns {{}}
 */
export function loadRulesConfigFromPresets(presetNames, moduleResolver) {
    if (presetNames === void 0) { presetNames = []; }
    var presetRulesConfig = {};
    presetNames.forEach(function (presetName) {
        var pkgPath = moduleResolver.resolvePresetPackageName(presetName);
        var preset = moduleInterop(require(pkgPath));
        if (!preset.hasOwnProperty("rules")) {
            throw new Error(presetName + " has not rules");
        }
        if (!preset.hasOwnProperty("rulesConfig")) {
            throw new Error(presetName + " has not rulesConfig");
        }
        // set config of <rule> to "<preset>/<rule>"
        Object.assign(presetRulesConfig, createFlatPresetRulesConfigFromRawPresetRuleConfig(preset.rulesConfig, presetName));
    });
    return presetRulesConfig;
}
//# sourceMappingURL=preset-loader.js.map