// LICENSE : MIT
"use strict";
/*
 textlint-core.js is a class
 textlint.js is a singleton object that is instance of textlint-core.js.
 */
import { TextlintKernelDescriptor } from "@textlint/kernel";
import { TextlintKernel } from "@textlint/kernel";
import { readFile } from "./util/fs-promise";
import { filterRulesObjectToKernelRule, pluginsObjectToKernelRule, rulesObjectToKernelRule } from "./util/object-to-kernel-format";
import textPlugin from "@textlint/textlint-plugin-text";
import markdownPlugin from "@textlint/textlint-plugin-markdown";
var path = require("path");
var throwIfTesting = require("@textlint/feature-flag").throwIfTesting;
/**
 * @class {TextLintCore}
 */
var TextLintCore = /** @class */ (function () {
    function TextLintCore(config) {
        if (config === void 0) { config = {}; }
        // this.config often is undefined.
        this.config = config;
        // Markdown and Text is enabled by default
        // Markdown and Text are for backward compatibility.
        this.defaultPlugins = [
            {
                pluginId: "markdown",
                plugin: markdownPlugin
            },
            {
                pluginId: "text",
                plugin: textPlugin
            }
        ];
        // TODO: remove `config`
        // https://github.com/textlint/textlint/issues/296
        this.kernel = new TextlintKernel(config);
        this.textlintKernelDescriptor = new TextlintKernelDescriptor({
            rules: [],
            plugins: this.defaultPlugins,
            filterRules: []
        });
    }
    /**
     * Use setupPlugins insteadof it.
     *
     * ````
     * textlint.setupPlugins({
     *   yourPluginName: yourPlugin
     * });
     * ````
     *
     * @param {*} Processor
     * @deprecated
     *
     * It will be removed until textlint@10
     */
    TextLintCore.prototype.addProcessor = function (Processor) {
        throwIfTesting("Use setupPlugins insteadof addProcessor method.`addProcessor` will be removed in the future." +
            "For more details, See https://github.com/textlint/textlint/issues/293");
        this.textlintKernelDescriptor = this.textlintKernelDescriptor.shallowMerge({
            plugins: [
                {
                    pluginId: "`${Processor.name}@deprecated`",
                    plugin: { Processor: Processor }
                }
            ].concat(this.defaultPlugins)
        });
    };
    /**
     * register Processors
     * @param {Object} plugins
     * @param {Object} [pluginsConfig]
     */
    TextLintCore.prototype.setupPlugins = function (plugins, pluginsConfig) {
        if (plugins === void 0) { plugins = {}; }
        if (pluginsConfig === void 0) { pluginsConfig = {}; }
        // Append default plugin to the plugins list.
        // Because, default plugin can be override by user plugins
        this.textlintKernelDescriptor = this.textlintKernelDescriptor.shallowMerge({
            plugins: pluginsObjectToKernelRule(plugins, pluginsConfig).concat(this.defaultPlugins)
        });
    };
    /**
     * Register rules and rulesConfig.
     * if want to release rules, please call {@link resetRules}.
     * @param {object} rules rule objects array
     * @param {object} [rulesOption] ruleConfig is object
     */
    TextLintCore.prototype.setupRules = function (rules, rulesOption) {
        if (rules === void 0) { rules = {}; }
        if (rulesOption === void 0) { rulesOption = {}; }
        this.textlintKernelDescriptor = this.textlintKernelDescriptor.shallowMerge({
            rules: rulesObjectToKernelRule(rules, rulesOption)
        });
    };
    /**
     * Register filterRules and filterRulesConfig.
     * if want to release rules, please call {@link resetRules}.
     * @param {object} filterRules rule objects array
     * @param {object} [filterRulesOption] ruleConfig is object
     */
    TextLintCore.prototype.setupFilterRules = function (filterRules, filterRulesOption) {
        if (filterRules === void 0) { filterRules = {}; }
        if (filterRulesOption === void 0) { filterRulesOption = {}; }
        this.textlintKernelDescriptor = this.textlintKernelDescriptor.shallowMerge({
            filterRules: filterRulesObjectToKernelRule(filterRules, filterRulesOption)
        });
    };
    /**
     * Remove all registered rule and clear messages.
     */
    TextLintCore.prototype.resetRules = function () {
        this.textlintKernelDescriptor = new TextlintKernelDescriptor({
            rules: [],
            plugins: this.defaultPlugins,
            filterRules: []
        });
    };
    /**
     * lint text by registered rules.
     * The result contains target filePath and error messages.
     * @param {string} text
     * @param {string} ext ext is extension. default: .txt
     * @returns {Promise.<TextlintResult>}
     */
    TextLintCore.prototype.lintText = function (text, ext) {
        if (ext === void 0) { ext = ".txt"; }
        var options = this._mergeSetupOptions({
            ext: ext
        });
        return this.kernel.lintText(text, options);
    };
    /**
     * lint markdown text by registered rules.
     * The result contains target filePath and error messages.
     * @param {string} text markdown format text
     * @returns {Promise.<TextlintResult>}
     */
    TextLintCore.prototype.lintMarkdown = function (text) {
        var ext = ".md";
        var options = this._mergeSetupOptions({
            ext: ext
        });
        return this.kernel.lintText(text, options);
    };
    /**
     * lint file and return result object
     * @param {string} filePath
     * @returns {Promise.<TextlintResult>} result
     */
    TextLintCore.prototype.lintFile = function (filePath) {
        var _this = this;
        var absoluteFilePath = path.resolve(process.cwd(), filePath);
        var ext = path.extname(absoluteFilePath);
        var options = this._mergeSetupOptions({
            ext: ext,
            filePath: absoluteFilePath
        });
        return readFile(absoluteFilePath).then(function (text) {
            return _this.kernel.lintText(text, options);
        });
    };
    /**
     * fix file and return fix result object
     * @param {string} filePath
     * @returns {Promise.<TextlintFixResult>}
     */
    TextLintCore.prototype.fixFile = function (filePath) {
        var _this = this;
        var absoluteFilePath = path.resolve(process.cwd(), filePath);
        var ext = path.extname(absoluteFilePath);
        var options = this._mergeSetupOptions({
            ext: ext,
            filePath: absoluteFilePath
        });
        return readFile(absoluteFilePath).then(function (text) {
            return _this.kernel.fixText(text, options);
        });
    };
    /**
     * fix texts and return fix result object
     * @param {string} text
     * @param {string} ext
     * @returns {Promise.<TextlintFixResult>}
     */
    TextLintCore.prototype.fixText = function (text, ext) {
        if (ext === void 0) { ext = ".txt"; }
        var options = this._mergeSetupOptions({
            ext: ext
        });
        return this.kernel.fixText(text, options);
    };
    /**
     * @private
     */
    TextLintCore.prototype._mergeSetupOptions = function (options) {
        var configFileBaseDir = typeof this.config.configFile === "string" ? path.dirname(this.config.configFile) : undefined;
        return Object.assign({}, options, {
            configBaseDir: configFileBaseDir,
            plugins: this.textlintKernelDescriptor.plugin.toKernelPluginsFormat(),
            rules: this.textlintKernelDescriptor.rule.toKernelRulesFormat(),
            filterRules: this.textlintKernelDescriptor.filterRule.toKernelFilterRulesFormat()
        });
    };
    return TextLintCore;
}());
export { TextLintCore };
//# sourceMappingURL=textlint-core.js.map