// LICENSE : MIT
"use strict";
var pathToGlob = require("path-to-glob-pattern");
var glob = require("glob");
var path = require("path");
var fs = require("fs");
var debug = require("debug")("textlint:find-util");
var DEFAULT_IGNORE_PATTERNS = Object.freeze(["**/.git/**", "**/node_modules/**"]);
var mapGitIgnorePatternTo = function (base) { return function (ignore) {
    if (ignore.startsWith("!")) {
        return "!" + path.posix.join(base, ignore.slice(1));
    }
    return path.posix.join(base, ignore);
}; };
var isFile = function (filePath) {
    try {
        return fs.statSync(filePath).isFile();
    }
    catch (error) {
        return false;
    }
};
/**
 * filter files by config
 * @param {string[]} patterns glob patterns
 * @param {{extensions?: string[], cwd?: string }} options
 */
export function pathsToGlobPatterns(patterns, options) {
    if (options === void 0) { options = {}; }
    var processPatterns = pathToGlob({
        extensions: options.extensions || [],
        cwd: options.cwd || process.cwd()
    });
    return patterns.map(processPatterns);
}
/**
 * found files by glob pattern
 * @param {string[]} patterns
 * @param {FindFilesOptions} options
 * @returns {string[]} file path list
 */
export function findFiles(patterns, options) {
    if (options === void 0) { options = {}; }
    var cwd = options.cwd || process.cwd();
    var ignoredPatterns = [];
    ignoredPatterns.push.apply(ignoredPatterns, DEFAULT_IGNORE_PATTERNS);
    if (options.ignoreFilePath) {
        var baseDir = path.resolve(cwd, path.dirname(options.ignoreFilePath));
        var normalizeIgnoreFilePath = path.resolve(cwd, options.ignoreFilePath);
        debug("findFiles ignore baseDir: %s, normalizeIgnoreFilePath: %s", baseDir, normalizeIgnoreFilePath);
        if (fs.existsSync(normalizeIgnoreFilePath)) {
            var ignored = fs
                .readFileSync(normalizeIgnoreFilePath, "utf-8")
                .split(/\r?\n/)
                .filter(function (line) { return !/^\s*$/.test(line) && !/^\s*#/.test(line); })
                .map(mapGitIgnorePatternTo(baseDir));
            debug("ignored: %o", ignored);
            ignoredPatterns.push.apply(ignoredPatterns, ignored);
        }
    }
    debug("search patterns: %o", patterns);
    debug("search ignore patterns: %o", ignoredPatterns);
    var files = [];
    var addFile = function (filePath) {
        if (files.indexOf(filePath) === -1) {
            files.push(filePath);
        }
    };
    patterns.forEach(function (pattern) {
        var file = path.resolve(cwd, pattern);
        if (isFile(file)) {
            addFile(fs.realpathSync(file));
        }
        else {
            glob.sync(pattern, {
                cwd: cwd,
                absolute: true,
                nodir: true,
                ignore: ignoredPatterns
            }).forEach(function (filePath) {
                // workaround for windows
                // https://github.com/isaacs/node-glob/issues/74#issuecomment-31548810
                addFile(path.resolve(filePath));
            });
        }
    });
    return files;
}
/**
 * @param {string[]} files
 * @param {{extensions?: string[]}} [options]
 * @returns {{availableFiles: string[], unAvailableFiles: string[]}}
 */
export function separateByAvailability(files, options) {
    if (options === void 0) { options = {}; }
    var extensions = options.extensions || [];
    var availableFiles = [];
    var unAvailableFiles = [];
    files.forEach(function (filePath) {
        var extname = path.extname(filePath);
        if (extensions.indexOf(extname) === -1) {
            unAvailableFiles.push(filePath);
        }
        else {
            availableFiles.push(filePath);
        }
    });
    return {
        availableFiles: availableFiles,
        unAvailableFiles: unAvailableFiles
    };
}
//# sourceMappingURL=find-util.js.map